/*
========================================

========================================
*/
fileIn "structs.ms"				-- supremecommander structs
filein "supcom2write.ms"	-- write to file functions

fn ExportSupCom2Format G path=
(
	format "\t*************\nexporting supcom2 model\n\t*************\n"
	--- check if gamemesh is multi surface.
	format "numsurfaces: %\n" G.numSurfaces
	if G.numSurfaces > 1 do
	(
		messagebox "GMError: More than 1 surface"
		return false
	)
	if G.isAnimated == false do
	(
		messagebox "GMError: not animated"
		return false
	)
	
	H = SC2_Header()
	H.infocount = 43				--- fixed string  "Exported with scmtool www.janvanderweg.com"
	
	Bonelist = SC2_Bonelist()
	numbones = G.Hierarchy.bones.count

	/* Collect bonelist */
	boneoffset = 64 -- (size of header)
	for i = 1 to numbones do
	(
		name = G.Hierarchy.bones[i].name
		append Bonelist.bones name
		append Bonelist.offset boneoffset
		boneoffset += (name.count + 1)
	)
	-- calculate padding and ensure there is room for 4 chars for the next FOURCC code
	bonepadding = 16 - (mod boneoffset 16)
	if bonepadding < 4 then	bonepadding += 16
	boneoffset += bonepadding
	bonepadding -= 4
	Bonelist.padding = bonepadding
	
	H.boneoffset = boneoffset
	H.wbonecount = G.Hierarchy.bones.count
	H.totalbones = G.Hierarchy.bones.count

	/* Collect bone data */
	sc2bones=#()
	for i = 1 to numbones do
	(
		sc2bone = SC2_Bone()
		format "bone: % parent: %\n" g.Hierarchy.bonename[i] g.hierarchy.parentindex[i]
		
		sc2bone.name = g.Hierarchy.bonename[i]
		sc2bone.nameoffset = Bonelist.offset[i]
		
		node = g.hierarchy.bones[i]
		parent = undefined
		parentindex = g.hierarchy.parentindex[i] + 1
		if parentindex != 0 do parent = g.hierarchy.bones[parentindex] \
		
		transform = node.transform
		parent = node.parent 
		if parent == undefined then
		(
			--local parentindex = -1
			sc2bone.parentboneindex = -1
			sc2bone.position = transform.row4
			sc2bone.rotation = quat 0 0 0 1
		)
		else
		(
			sc2bone.parentboneindex = g.hierarchy.parentindex[i]
			--relative position and rotation
			sc2bone.position = ( in coordsys parent node.pos )
			sc2bone.rotation = ( in coordsys parent node.rotation ) 
		)
		Transform.row1 = normalize Transform.row1
		Transform.row2 = normalize Transform.row2
		Transform.row3 = normalize Transform.row3
		iTransform = inverse transform
		sc2bone.RestPoseInverse1 = ( normalize [ iTransform.row1[1], iTransform.row2[1], iTransform.row3[1]] )
		sc2bone.RestPoseInverse2 = ( normalize [ iTransform.row1[2], iTransform.row2[2], iTransform.row3[2]] )
		sc2bone.RestPoseInverse3 = ( normalize [ iTransform.row1[3], iTransform.row2[3], iTransform.row3[3]] )
		sc2bone.RestPoseInverse4 = iTransform.row4
		
		append sc2bones sc2bone
	)
	-- calculate padding
	--a single bone entry is 108 bytes. 27*4
	vertexoffset = H.boneoffset + ( H.totalbones * 108 )
	bonepadding = 16 - ( mod vertexoffset 16 )
	if bonepadding < 4 then
	(
		bonepadding += 16
	)
	vertexoffset +=bonepadding
	bonepadding -= 4
	H.vertoffset = vertexoffset
	BoneDataPadding = bonepadding
	
	/* Vertex data */
	s = G.Surfaces[1] -- assume only one surface
	sc2verts=#()
	
	H.indexcount = s.numtris * 3
	H.vertcount = s.numverts
	
	for i = 1 to s.numverts do
	(
		sc2vert = SC2_Vertex()
		gmvert = s.Vertex[i]
		
		sc2vert.position = gmvert.position
		sc2vert.normal = gmvert.normal
		sc2vert.tangent = gmvert.tangent
		sc2vert.bitangent = gmvert.bitangent
		sc2vert.uv1 = gmvert.uvw
		sc2vert.uv2 = gmvert.uvw
		
		bl=#()
		wl=#()
		tw = 0
		for j = 1 to 4 do
		(
			b = gmvert.boneindex[j]
			w = gmvert.boneweight[j]
			b = finditem G.Hierarchy.bonename b
			b -= 1
			if gmvert.boneindex[j] == undefined do b = 0
			w = (w *255) as integer
			tw += w
			append bl b
			append wl w
		)
		if tw < 255 do	--ensure that all weights total 255
		(
			toadd = 255 - tw
			wl[1] += toadd
		)
		sc2vert.boneindex = bl
		sc2vert.boneweights = wl
		append sc2verts sc2vert
	)
	
	--calculate vertexdata padding
	--a single vertex entry is 68 bytes. 17*4
	indexoffset = H.vertoffset + ( sc2verts.count * 68 )
	vertdatapadding = 16 - ( mod indexoffset 16 )
	if vertdatapadding < 4 then
	(
		vertdatapadding += 16
	)
	indexoffset = ( indexoffset + vertdatapadding )
	vertdatapadding -= 4
	H.indexoffset = indexoffset
	VertexDataPadding = vertdatapadding
	
	--calculate trisdata padding
	--a single index entry is 6 bytes. 3*2
	xtraoffset = H.indexoffset + ( s.numtris * 6 )
	trispadding = 16 - ( mod xtraoffset 16 )
	if trispadding < 4 then
	(
		trispadding += 16
	)
	xtraoffset += trispadding
	trispadding -= 4
	H.xtraoffset = xtraoffset
	TrisDataPadding = trispadding
	
	-- calculate xtra data padding
	-- a single xtra data entry is 4 bytes 
	infooffset = H.xtraoffset + (sc2verts.count * 4)
	xtrapadding = 16 - ( mod infooffset 16)
	if xtrapadding < 4 then
	(
		xtrapadding += 16
	)
	infooffset += xtrapadding
	xtrapadding -= 4
	H.infooffset = infooffset
	XtraDataPadding = xtrapadding
	
	H.materialoffset = H.infooffset + 96
	H.materialnum = 1
	
	/* Write file */
	bstream = fopen path "wb"

	SC2_WriteHeader bstream H
	SC2_WriteBonelist bstream Bonelist
	SC2_WriteBones bstream sc2bones BoneDataPadding
	SC2_WriteVerts bstream sc2verts VertexDataPadding
	SC2_WriteTris bstream s.tris TrisDataPadding
	SC2_WriteXtra bstream sc2verts XtraDataPadding
	
	WriteString bstream "Exported with scmtool www.janvanderweg.com"
	for i = 1 to 33 do WriteByte bstream 187
	WriteByte bstream 77 -- MNAM
	WriteByte bstream 78
	WriteByte bstream 65
	WriteByte bstream 77
	WriteString bstream "Diffuse"
	for i = 1 to 4 do WriteByte bstream 187
	WriteByte bstream 77 -- MATS
	WriteByte bstream 65
	WriteByte bstream 84
	WriteByte bstream 83
	
	WriteLong bstream (H.materialoffset - 16)
	WriteLong bstream s.numtris 
	WriteLong bstream 0 
	WriteLong bstream 0
	
	WriteLong bstream s.numverts
	
	fclose bstream
)
